@@ -18,7 +18,7 @@
 #
 #
 
-from typing import Callable, Optional
+from typing import Callable
 
 import attr
 
@@ -503,7 +503,7 @@ class RoomVersionCapability:
     """An object which describes the unique attributes of a room version."""
 
     identifier: str  # the identifier for this capability
-    preferred_version: Optional[RoomVersion]
+    preferred_version: RoomVersion | None
     support_check_lambda: Callable[[RoomVersion], bool]
 
 
