@@ -21,7 +21,7 @@
 import html
 import logging
 import urllib.parse
-from typing import TYPE_CHECKING, Optional, cast
+from typing import TYPE_CHECKING, cast
 
 import attr
 
@@ -42,12 +42,12 @@ class OEmbedResult:
     # The Open Graph result (converted from the oEmbed result).
     open_graph_result: JsonDict
     # The author_name of the oEmbed result
-    author_name: Optional[str]
+    author_name: str | None
     # Number of milliseconds to cache the content, according to the oEmbed response.
     #
     # This will be None if no cache-age is provided in the oEmbed response (or
     # if the oEmbed response cannot be turned into an Open Graph response).
-    cache_age: Optional[int]
+    cache_age: int | None
 
 
 class OEmbedProvider:
@@ -80,7 +80,7 @@ class OEmbedProvider:
             for pattern in oembed_endpoint.url_patterns:
                 self._oembed_patterns[pattern] = api_endpoint
 
-    def get_oembed_url(self, url: str) -> Optional[str]:
+    def get_oembed_url(self, url: str) -> str | None:
         """
         Check whether the URL should be downloaded as oEmbed content instead.
 
@@ -105,7 +105,7 @@ class OEmbedProvider:
         # No match.
         return None
 
-    def autodiscover_from_html(self, tree: "etree._Element") -> Optional[str]:
+    def autodiscover_from_html(self, tree: "etree._Element") -> str | None:
         """
         Search an HTML document for oEmbed autodiscovery information.
 
