@@ -20,7 +20,7 @@
 #
 
 import logging
-from typing import TYPE_CHECKING, Any, Optional
+from typing import TYPE_CHECKING, Any
 
 from prometheus_client import Counter, Histogram
 
@@ -73,7 +73,7 @@ class ExternalCache:
         self.server_name = hs.hostname
 
         if hs.config.redis.redis_enabled:
-            self._redis_connection: Optional["ConnectionHandler"] = (
+            self._redis_connection: "ConnectionHandler" | None = (
                 hs.get_outbound_redis_connection()
             )
         else:
@@ -121,7 +121,7 @@ class ExternalCache:
                     )
                 )
 
-    async def get(self, cache_name: str, key: str) -> Optional[Any]:
+    async def get(self, cache_name: str, key: str) -> Any | None:
         """Look up a key/value in the named cache."""
 
         if self._redis_connection is None:
