@@ -29,7 +29,6 @@ from typing import (
     Awaitable,
     Callable,
     Iterable,
-    Optional,
     Sequence,
     cast,
 )
@@ -41,6 +40,7 @@ from synapse.storage.engines import PostgresEngine
 from synapse.storage.types import Connection, Cursor
 from synapse.types import JsonDict, StrCollection
 from synapse.util.clock import Clock
+from synapse.util.duration import Duration
 from synapse.util.json import json_encoder
 
 from . import engines
@@ -163,15 +163,15 @@ class _BackgroundUpdateContextManager:
 
     async def __aenter__(self) -> int:
         if self._sleep:
-            await self._clock.sleep(self._sleep_duration_ms / 1000)
+            await self._clock.sleep(Duration(milliseconds=self._sleep_duration_ms))
 
         return self._update_duration_ms
 
     async def __aexit__(
         self,
-        exc_type: Optional[type[BaseException]],
-        exc: Optional[BaseException],
-        tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc: BaseException | None,
+        tb: TracebackType | None,
     ) -> None:
         pass
 
@@ -196,7 +196,7 @@ class BackgroundUpdatePerformance:
         self.avg_item_count += 0.1 * (item_count - self.avg_item_count)
         self.avg_duration_ms += 0.1 * (duration_ms - self.avg_duration_ms)
 
-    def average_items_per_ms(self) -> Optional[float]:
+    def average_items_per_ms(self) -> float | None:
         """An estimate of how long it takes to do a single update.
         Returns:
             A duration in ms as a float
@@ -212,7 +212,7 @@ class BackgroundUpdatePerformance:
             # changes in how long the update process takes.
             return float(self.avg_item_count) / float(self.avg_duration_ms)
 
-    def total_items_per_ms(self) -> Optional[float]:
+    def total_items_per_ms(self) -> float | None:
         """An estimate of how long it takes to do a single update.
         Returns:
             A duration in ms as a float
@@ -250,11 +250,11 @@ class BackgroundUpdater:
         self._database_name = database.name()
 
         # if a background update is currently running, its name.
-        self._current_background_update: Optional[str] = None
+        self._current_background_update: str | None = None
 
-        self._on_update_callback: Optional[ON_UPDATE_CALLBACK] = None
-        self._default_batch_size_callback: Optional[DEFAULT_BATCH_SIZE_CALLBACK] = None
-        self._min_batch_size_callback: Optional[MIN_BATCH_SIZE_CALLBACK] = None
+        self._on_update_callback: ON_UPDATE_CALLBACK | None = None
+        self._default_batch_size_callback: DEFAULT_BATCH_SIZE_CALLBACK | None = None
+        self._min_batch_size_callback: MIN_BATCH_SIZE_CALLBACK | None = None
 
         self._background_update_performance: dict[str, BackgroundUpdatePerformance] = {}
         self._background_update_handlers: dict[str, _BackgroundUpdateHandler] = {}
@@ -304,8 +304,8 @@ class BackgroundUpdater:
     def register_update_controller_callbacks(
         self,
         on_update: ON_UPDATE_CALLBACK,
-        default_batch_size: Optional[DEFAULT_BATCH_SIZE_CALLBACK] = None,
-        min_batch_size: Optional[DEFAULT_BATCH_SIZE_CALLBACK] = None,
+        default_batch_size: DEFAULT_BATCH_SIZE_CALLBACK | None = None,
+        min_batch_size: DEFAULT_BATCH_SIZE_CALLBACK | None = None,
     ) -> None:
         """Register callbacks from a module for each hook."""
         if self._on_update_callback is not None:
@@ -380,7 +380,7 @@ class BackgroundUpdater:
 
         return self.minimum_background_batch_size
 
-    def get_current_update(self) -> Optional[BackgroundUpdatePerformance]:
+    def get_current_update(self) -> BackgroundUpdatePerformance | None:
         """Returns the current background update, if any."""
 
         update_name = self._current_background_update
@@ -526,14 +526,14 @@ class BackgroundUpdater:
             True if we have finished running all the background updates, otherwise False
         """
 
-        def get_background_updates_txn(txn: Cursor) -> list[tuple[str, Optional[str]]]:
+        def get_background_updates_txn(txn: Cursor) -> list[tuple[str, str | None]]:
             txn.execute(
                 """
                 SELECT update_name, depends_on FROM background_updates
                 ORDER BY ordering, update_name
                 """
             )
-            return cast(list[tuple[str, Optional[str]]], txn.fetchall())
+            return cast(list[tuple[str, str | None]], txn.fetchall())
 
         if not self._current_background_update:
             all_pending_updates = await self.db_pool.runInteraction(
@@ -669,10 +669,10 @@ class BackgroundUpdater:
         index_name: str,
         table: str,
         columns: Iterable[str],
-        where_clause: Optional[str] = None,
+        where_clause: str | None = None,
         unique: bool = False,
         psql_only: bool = False,
-        replaces_index: Optional[str] = None,
+        replaces_index: str | None = None,
     ) -> None:
         """Helper for store classes to do a background index addition
 
@@ -763,10 +763,10 @@ class BackgroundUpdater:
         index_name: str,
         table: str,
         columns: Iterable[str],
-        where_clause: Optional[str] = None,
+        where_clause: str | None = None,
         unique: bool = False,
         psql_only: bool = False,
-        replaces_index: Optional[str] = None,
+        replaces_index: str | None = None,
     ) -> None:
         """Add an index in the background.
 
@@ -862,7 +862,7 @@ class BackgroundUpdater:
                 c.execute(sql)
 
         if isinstance(self.db_pool.engine, engines.PostgresEngine):
-            runner: Optional[Callable[[LoggingDatabaseConnection], None]] = (
+            runner: Callable[[LoggingDatabaseConnection], None] | None = (
                 create_index_psql
             )
         elif psql_only:
