@@ -25,10 +25,6 @@ from typing import (
     TYPE_CHECKING,
     Collection,
     Iterable,
-    List,
-    Optional,
-    Tuple,
-    Union,
     cast,
 )
 
@@ -59,16 +55,16 @@ logger = logging.getLogger(__name__)
 class LocalMedia:
     media_id: str
     media_type: str
-    media_length: Optional[int]
+    media_length: int | None
     upload_name: str
     created_ts: int
-    url_cache: Optional[str]
+    url_cache: str | None
     last_access_ts: int
-    quarantined_by: Optional[str]
+    quarantined_by: str | None
     safe_from_quarantine: bool
-    user_id: Optional[str]
-    authenticated: Optional[bool]
-    sha256: Optional[str]
+    user_id: str | None
+    authenticated: bool | None
+    sha256: str | None
 
 
 @attr.s(slots=True, frozen=True, auto_attribs=True)
@@ -77,20 +73,20 @@ class RemoteMedia:
     media_id: str
     media_type: str
     media_length: int
-    upload_name: Optional[str]
+    upload_name: str | None
     filesystem_id: str
     created_ts: int
     last_access_ts: int
-    quarantined_by: Optional[str]
-    authenticated: Optional[bool]
-    sha256: Optional[str]
+    quarantined_by: str | None
+    authenticated: bool | None
+    sha256: str | None
 
 
 @attr.s(slots=True, frozen=True, auto_attribs=True)
 class UrlCache:
     response_code: int
     expires_ts: int
-    og: Union[str, bytes]
+    og: str | bytes
 
 
 class MediaSortOrder(Enum):
@@ -185,7 +181,7 @@ class MediaRepositoryBackgroundUpdateStore(SQLBaseStore):
         )
 
         if hs.config.media.can_load_media_repo:
-            self.unused_expiration_time: Optional[int] = (
+            self.unused_expiration_time: int | None = (
                 hs.config.media.unused_expiration_time
             )
         else:
@@ -226,7 +222,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         super().__init__(database, db_conn, hs)
         self.server_name: str = hs.hostname
 
-    async def get_local_media(self, media_id: str) -> Optional[LocalMedia]:
+    async def get_local_media(self, media_id: str) -> LocalMedia | None:
         """Get the metadata for a local piece of media
 
         Returns:
@@ -275,7 +271,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         user_id: str,
         order_by: str = MediaSortOrder.CREATED_TS.value,
         direction: Direction = Direction.FORWARDS,
-    ) -> Tuple[List[LocalMedia], int]:
+    ) -> tuple[list[LocalMedia], int]:
         """Get a paginated list of metadata for a local piece of media
         which an user_id has uploaded
 
@@ -292,7 +288,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
 
         def get_local_media_by_user_paginate_txn(
             txn: LoggingTransaction,
-        ) -> Tuple[List[LocalMedia], int]:
+        ) -> tuple[list[LocalMedia], int]:
             # Set ordering
             order_by_column = MediaSortOrder(order_by).value
 
@@ -301,14 +297,14 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             else:
                 order = "ASC"
 
-            args: List[Union[str, int]] = [user_id]
+            args: list[str | int] = [user_id]
             sql = """
                 SELECT COUNT(*) as total_media
                 FROM local_media_repository
                 WHERE user_id = ?
             """
             txn.execute(sql, args)
-            count = cast(Tuple[int], txn.fetchone())[0]
+            count = cast(tuple[int], txn.fetchone())[0]
 
             sql = """
                 SELECT
@@ -365,7 +361,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         keep_profiles: bool,
         include_quarantined_media: bool,
         include_protected_media: bool,
-    ) -> List[str]:
+    ) -> list[str]:
         """
         Retrieve a list of media IDs from the local media store.
 
@@ -437,7 +433,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
                 AND NOT safe_from_quarantine
             """
 
-        def _get_local_media_ids_txn(txn: LoggingTransaction) -> List[str]:
+        def _get_local_media_ids_txn(txn: LoggingTransaction) -> list[str]:
             txn.execute(sql, (before_ts, before_ts, size_gt))
             return [row[0] for row in txn]
 
@@ -474,12 +470,12 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         media_id: str,
         media_type: str,
         time_now_ms: int,
-        upload_name: Optional[str],
+        upload_name: str | None,
         media_length: int,
         user_id: UserID,
-        url_cache: Optional[str] = None,
-        sha256: Optional[str] = None,
-        quarantined_by: Optional[str] = None,
+        url_cache: str | None = None,
+        sha256: str | None = None,
+        quarantined_by: str | None = None,
     ) -> None:
         if self.hs.config.media.enable_authenticated_media:
             authenticated = True
@@ -507,12 +503,12 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         self,
         media_id: str,
         media_type: str,
-        upload_name: Optional[str],
+        upload_name: str | None,
         media_length: int,
         user_id: UserID,
         sha256: str,
-        url_cache: Optional[str] = None,
-        quarantined_by: Optional[str] = None,
+        url_cache: str | None = None,
+        quarantined_by: str | None = None,
     ) -> None:
         updatevalues = {
             "media_type": media_type,
@@ -544,7 +540,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             desc="mark_local_media_as_safe",
         )
 
-    async def count_pending_media(self, user_id: UserID) -> Tuple[int, int]:
+    async def count_pending_media(self, user_id: UserID) -> tuple[int, int]:
         """Count the number of pending media for a user.
 
         Returns:
@@ -552,7 +548,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             expiration timestamp.
         """
 
-        def get_pending_media_txn(txn: LoggingTransaction) -> Tuple[int, int]:
+        def get_pending_media_txn(txn: LoggingTransaction) -> tuple[int, int]:
             sql = """
             SELECT COUNT(*), MIN(created_ts)
             FROM local_media_repository
@@ -577,13 +573,13 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             "get_pending_media", get_pending_media_txn
         )
 
-    async def get_url_cache(self, url: str, ts: int) -> Optional[UrlCache]:
+    async def get_url_cache(self, url: str, ts: int) -> UrlCache | None:
         """Get the media_id and ts for a cached URL as of the given timestamp
         Returns:
             None if the URL isn't cached.
         """
 
-        def get_url_cache_txn(txn: LoggingTransaction) -> Optional[UrlCache]:
+        def get_url_cache_txn(txn: LoggingTransaction) -> UrlCache | None:
             # get the most recently cached result (relative to the given ts)
             sql = """
                 SELECT response_code, expires_ts, og
@@ -617,7 +613,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         self,
         url: str,
         response_code: int,
-        etag: Optional[str],
+        etag: str | None,
         expires_ts: int,
         og: str,
         media_id: str,
@@ -637,9 +633,9 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             desc="store_url_cache",
         )
 
-    async def get_local_media_thumbnails(self, media_id: str) -> List[ThumbnailInfo]:
+    async def get_local_media_thumbnails(self, media_id: str) -> list[ThumbnailInfo]:
         rows = cast(
-            List[Tuple[int, int, str, str, int]],
+            list[tuple[int, int, str, str, int]],
             await self.db_pool.simple_select_list(
                 "local_media_repository_thumbnails",
                 {"media_id": media_id},
@@ -685,7 +681,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
 
     async def get_cached_remote_media(
         self, origin: str, media_id: str
-    ) -> Optional[RemoteMedia]:
+    ) -> RemoteMedia | None:
         row = await self.db_pool.simple_select_one(
             "remote_media_cache",
             {"media_origin": origin, "media_id": media_id},
@@ -726,9 +722,9 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         media_type: str,
         media_length: int,
         time_now_ms: int,
-        upload_name: Optional[str],
+        upload_name: str | None,
         filesystem_id: str,
-        sha256: Optional[str],
+        sha256: str | None,
     ) -> None:
         if self.hs.config.media.enable_authenticated_media:
             authenticated = True
@@ -755,7 +751,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
     async def update_cached_last_access_time(
         self,
         local_media: Iterable[str],
-        remote_media: Iterable[Tuple[str, str]],
+        remote_media: Iterable[tuple[str, str]],
         time_ms: int,
     ) -> None:
         """Updates the last access time of the given media
@@ -793,9 +789,9 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
 
     async def get_remote_media_thumbnails(
         self, origin: str, media_id: str
-    ) -> List[ThumbnailInfo]:
+    ) -> list[ThumbnailInfo]:
         rows = cast(
-            List[Tuple[int, int, str, str, int]],
+            list[tuple[int, int, str, str, int]],
             await self.db_pool.simple_select_list(
                 "remote_media_cache_thumbnails",
                 {"media_origin": origin, "media_id": media_id},
@@ -824,7 +820,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
         t_width: int,
         t_height: int,
         t_type: str,
-    ) -> Optional[ThumbnailInfo]:
+    ) -> ThumbnailInfo | None:
         """Fetch the thumbnail info of given width, height and type."""
 
         row = await self.db_pool.simple_select_one(
@@ -881,7 +877,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
 
     async def get_remote_media_ids(
         self, before_ts: int, include_quarantined_media: bool
-    ) -> List[Tuple[str, str, str]]:
+    ) -> list[tuple[str, str, str]]:
         """
         Retrieve a list of server name, media ID tuples from the remote media cache.
 
@@ -911,7 +907,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             """
 
         return cast(
-            List[Tuple[str, str, str]],
+            list[tuple[str, str, str]],
             await self.db_pool.execute("get_remote_media_ids", sql, before_ts),
         )
 
@@ -932,7 +928,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             "delete_remote_media", delete_remote_media_txn
         )
 
-    async def get_expired_url_cache(self, now_ts: int) -> List[str]:
+    async def get_expired_url_cache(self, now_ts: int) -> list[str]:
         sql = (
             "SELECT media_id FROM local_media_repository_url_cache"
             " WHERE expires_ts < ?"
@@ -940,7 +936,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             " LIMIT 500"
         )
 
-        def _get_expired_url_cache_txn(txn: LoggingTransaction) -> List[str]:
+        def _get_expired_url_cache_txn(txn: LoggingTransaction) -> list[str]:
             txn.execute(sql, (now_ts,))
             return [row[0] for row in txn]
 
@@ -959,7 +955,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
 
         await self.db_pool.runInteraction("delete_url_cache", _delete_url_cache_txn)
 
-    async def get_url_cache_media_before(self, before_ts: int) -> List[str]:
+    async def get_url_cache_media_before(self, before_ts: int) -> list[str]:
         sql = (
             "SELECT media_id FROM local_media_repository"
             " WHERE created_ts < ? AND url_cache IS NOT NULL"
@@ -967,7 +963,7 @@ class MediaRepositoryStore(MediaRepositoryBackgroundUpdateStore):
             " LIMIT 500"
         )
 
-        def _get_url_cache_media_before_txn(txn: LoggingTransaction) -> List[str]:
+        def _get_url_cache_media_before_txn(txn: LoggingTransaction) -> list[str]:
             txn.execute(sql, (before_ts,))
             return [row[0] for row in txn]
 
