@@ -28,8 +28,6 @@ from typing import (
     Any,
     Collection,
     Iterable,
-    Optional,
-    Union,
     cast,
 )
 
@@ -60,7 +58,7 @@ class SearchEntry:
     value: str
     event_id: str
     room_id: str
-    stream_ordering: Optional[int]
+    stream_ordering: int | None
     origin_server_ts: int
 
 
@@ -516,7 +514,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (rank, room_id, event_id).
         results = cast(
-            list[tuple[Union[int, float], str, str]],
+            list[tuple[int | float, str, str]],
             await self.db_pool.execute("search_msgs", sql, *args),
         )
 
@@ -562,7 +560,7 @@ class SearchStore(SearchBackgroundUpdateStore):
         search_term: str,
         keys: Iterable[str],
         limit: int,
-        pagination_token: Optional[str] = None,
+        pagination_token: str | None = None,
     ) -> JsonDict:
         """Performs a full text search over events with given keys.
 
@@ -683,7 +681,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (rank, room_id, event_id, origin_server_ts, stream_ordering).
         results = cast(
-            list[tuple[Union[int, float], str, str, int, int]],
+            list[tuple[int | float, str, str, int, int]],
             await self.db_pool.execute("search_rooms", sql, *args),
         )
 
@@ -817,7 +815,7 @@ class SearchToken(enum.Enum):
     And = enum.auto()
 
 
-Token = Union[str, Phrase, SearchToken]
+Token = str | Phrase | SearchToken
 TokenList = list[Token]
 
 
