@@ -58,6 +58,7 @@ from synapse.logging.context import (
     run_in_background,
 )
 from synapse.util.clock import Clock
+from synapse.util.duration import Duration
 
 logger = logging.getLogger(__name__)
 
@@ -640,7 +641,7 @@ class Linearizer:
         # This needs to happen while we hold the lock. We could put it on the
         # exit path, but that would slow down the uncontended case.
         try:
-            await self._clock.sleep(0)
+            await self._clock.sleep(Duration(seconds=0))
         except CancelledError:
             self._release_lock(key, entry)
             raise
@@ -818,7 +819,9 @@ def timeout_deferred(
 
     # We don't track these calls since they are short.
     delayed_call = clock.call_later(
-        timeout, time_it_out, call_later_cancel_on_shutdown=cancel_on_shutdown
+        Duration(seconds=timeout),
+        time_it_out,
+        call_later_cancel_on_shutdown=cancel_on_shutdown,
     )
 
     def convert_cancelled(value: Failure) -> Failure:
